package cc.fascinated.bat.service;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.DiscordMessage;
import cc.fascinated.bat.repository.DiscordMessageRepository;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.events.message.MessageDeleteEvent;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.events.message.MessageUpdateEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Service;

import java.util.Optional;

/**
 * @author Fascinated (fascinated7)
 */
@Service
@Log4j2(topic = "Discord Message Service")
@DependsOn("discordService")
public class DiscordMessageService extends ListenerAdapter {
    private final DiscordMessageRepository discordMessageRepository;
    private final GuildService guildService;
    private final UserService userService;

    @Autowired
    public DiscordMessageService(DiscordMessageRepository discordMessageRepository, @NonNull GuildService guildService,
                                 @NonNull UserService userService) {
        this.discordMessageRepository = discordMessageRepository;
        this.guildService = guildService;
        this.userService = userService;

        DiscordService.JDA.addEventListener(this);
    }

    /**
     * Gets the message with the given id
     *
     * @param messageId the id of the message
     * @return the message with the given id
     */
    public DiscordMessage getMessage(String messageId) {
        Optional<DiscordMessage> optionalMessage = discordMessageRepository.findById(messageId);
        return optionalMessage.orElse(null);
    }

    @Override
    public void onMessageReceived(@NotNull MessageReceivedEvent event) {
        discordMessageRepository.save(new DiscordMessage(
                event.getMessageId(),
                event.getMessage().getTimeCreated().toInstant().toEpochMilli(),
                event.getChannel().getId(),
                event.getGuild().getId(),
                event.getAuthor().getId(),
                event.getMessage().getContentStripped(),
                false
        ));
    }

    @Override
    public void onMessageUpdate(@NotNull MessageUpdateEvent event) {
        Optional<DiscordMessage> message = discordMessageRepository.findById(event.getMessageId());
        DiscordMessage oldMessage = message.orElse(null);
        if (oldMessage != null) {
            if (oldMessage.getContent().equals(event.getMessage().getContentStripped())) {
                return;
            }
            discordMessageRepository.delete(oldMessage);
        }
        DiscordMessage newMessage = new DiscordMessage(
                event.getMessageId(),
                event.getMessage().getTimeCreated().toInstant().toEpochMilli(),
                event.getChannel().getId(),
                event.getGuild().getId(),
                event.getAuthor().getId(),
                event.getMessage().getContentStripped(),
                false
        );
        discordMessageRepository.save(newMessage);

        BatGuild guild = guildService.getGuild(event.getGuild().getId());
        BatUser user = userService.getUser(event.getAuthor().getId());
        if (guild == null || user == null) {
            return;
        }
        for (EventListener listener : EventService.LISTENERS) {
            listener.onGuildMessageEdit(guild, user, oldMessage, newMessage, event);
        }
    }

    @Override
    public void onMessageDelete(@NotNull MessageDeleteEvent event) {
        Optional<DiscordMessage> optionalMessage = discordMessageRepository.findById(event.getMessageId());
        if (optionalMessage.isEmpty()) {
            return;
        }
        DiscordMessage message = optionalMessage.get();
        message.setDeleted(true);
        discordMessageRepository.save(message);
    }
}
